<?php defined('SYSPATH') or die('No direct script access.');

/**
 * Class Model_Discipline
 *
 * @property-read $ID int
 * @property $authorID int
 * @property $gradeID int
 * @property $facultyID int
 * @property $subjectID int
 * @property $subjectName
 * @property $semesterID int
 * @property $lectures int
 * @property $practice int
 * @property $labs int
 * @property $type string
 * @property $isLocked bool
 */
class Model_Discipline extends Model implements Serializable
{
    private $data;

    /**
     * @param $data array
     * Source data for building the object.
     * @param $associated bool
     * Indicates whether object exists in db.
     */
    function __construct(array & $data, $associated = true) {
        $this->data = $data;
        if (!$associated || !isset($data['ID']))
            $this->create();
    }

    /**
     * Creation of new discipline from raw data.
     * @return Model_Helper_DisciplineBuilder
     */
    static public function make() {
        return new Model_Helper_DisciplineBuilder();
    }

    /**
     * Load the discipline from database.
     * @param $id int  discipline id
     * @return $this  new instance
     */
    static public function load($id) {
        if ($id <= 0)
            throw new InvalidArgumentException(Error::ID_IS_INCORRECT);

        // todo: lazy loading
//        throw new LogicException('Not Implemented Yet');
        return new self($data = self::getInfo($id), true);
    }


    /**
     * Create new discipline in db, based on $data.
     */
    private function create() {
        $sql = "SELECT `Discipline_Create`(authorID, gradeID, subjectID, type, lectures, practice, labs, facultyID, semesterID, NULL) AS `Num`;";

        $this->data['ID'] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()[0]['Num'];

        $this->data['subtype'] = null;
        $this->data['isLocked'] = false;
//        $this->data['milestone'] = 0;     # todo: what is it?
    }

    public function delete() {
        $sql = "SELECT `Discipline_Delete`(:id);";
        DB::query(Database::SELECT, $sql)->param(':id', $this->ID)->execute();
    }

    function __get($name) {
        if (isset($this->data[$name]))
            return $this->data[$name];
        throw new InvalidArgumentException;
    }

    /**
     * String representation of object.
     * @return string  the string representation of the object or null
     */
    public function serialize() {
        return serialize($this->data);
    }

    /**
     * Constructs the object
     * @param string $serialized
     * The string representation of the object.
     * @return void
     */
    public function unserialize($serialized) {
        $this->data = unserialize($serialized);
    }

    /**
     * @param $id int discipline id
     * @return array data from <tt>view_disciplines</tt> table
     * @throws HTTP_Exception if discipline does not exist
     */
    public static function getInfo($id) {
        $sql = "CALL `GetDisciplineInfo`('$id'); ";
        $info = DB::query(Database::SELECT, $sql)->execute();

        if($info->count() == 0)
            throw HTTP_Exception::factory(404, "Учебная карта дисциплины с ID $id не найдена!");
        return $info->offsetGet(0);
    }


    /** Change grade. */
    public static function changeGrade($DisciplineID, $teacherID, $Grade) {
        $sql = "SELECT `ChangeDisciplineGrade`('$teacherID', '$DisciplineID', '$Grade') AS `Num`;";
        return DB::query(Database::SELECT, $sql)->execute();
    }

    /**
     * Check, whether discipline already exists.
     *
     * @param $teacher int teacher id
     * @param $subject int subject id
     * @param $grade int grade num (1-7)
     * @return true, if exists
     */
    public static function exists($teacher, $subject, $grade) {
        $disciplines = Model_Disciplines::ofTeacher($teacher);
        foreach ($disciplines as $d) {
            if ($subject == $d['SubjectID'] && $grade == $d['GradeNum'])
                return true;
        }
        return false;
    }


    /**
     * @param $teacherID int
     * @param $disciplineID int
     * @param $milestone int
     * @return Database_Result
     */
    public static function setMilestone($teacherID, $disciplineID, $milestone) {
        $sql = "SELECT `RestrictAfterMilestone`('$teacherID', '$disciplineID', '$milestone') AS 'Num'";
        return DB::query(Database::SELECT, $sql)->execute();
    }

    /**
     * @param $facultyID int
     * @param $semesterID int
     * @return Database_Result
     */
    public static function getMilestone($facultyID, $semesterID) {
        $sql = "SELECT `GetMilestone`('$facultyID', '$semesterID') AS 'Num'";
        return DB::query(Database::SELECT, $sql)->execute();
    }


    /**
     * @param $teacherID int
     * @param $facultyID int
     * @param $milestone int
     * @return Database_Result
     */
    public static function setMilestoneForCredits($teacherID, $facultyID, $milestone) {
        $sql = "SELECT `RestrictAfterMilestoneForCredits`('$teacherID', '$facultyID', '$milestone') AS 'Num'";
        return DB::query(Database::SELECT, $sql)->execute();
    }

}